delegate void Notify();

class Move {
    public int x, y;

    public Move(int x, int y) {
        this.x = x; this.y = y;
    }
}

class Game {
    int[,] square = new int[3, 3];    // 0 = empty, 1 = player 1, 2 = player 2
    int moves = 0;  // number of moves on board

    public int turn = 1;     // whose turn it is
    public int won = 0;     // who has won
    public int win_x, win_y, win_dx, win_dy;    // vector indicating winning squares
    
    public event Notify? changed;    // fires whenever the board changes
    
    public Game clone() {
        Game g = new Game();
        g.square = (int[,]) square.Clone();
        g.moves = moves;
        g.turn = turn;
        g.won = won;
        return g;
    }

    public int this[int x, int y] {
        get => square[x, y];
    }

    public int? winner() {
        if (won != 0)
            return won;
        if (moves == 9)
            return 0;
        return null;
    }

    // Return true if the current player has three in a row,
    // starting at (x, y) and moving in direction (dx, dy).
    bool all(int x, int y, int dx, int dy) {
        for (int i = 0 ; i < 3 ; ++i) 
            if (square[x + i * dx, y + i * dy] != turn)
                return false;
        
        win_x = x; win_y = y; win_dx = dx; win_dy = dy;
        return true;
    }

    // Return true if the current player has won.
    bool checkWin() {
        // check rows and columns
        for (int i = 0 ; i < 3 ; ++i)
            if (all(i, 0, 0, 1) || all(0, i, 1, 0))
                return true;
                
        // check diagonals
        return all(0, 0, 1, 1) || all(2, 0, -1, 1);
    }
    
    // Make a move.  Return true if the move was legal.
    public bool move(Move m) {
        if (square[m.x, m.y] > 0)    // square is occupied
            return false;
            
        square[m.x, m.y] = turn;
        moves += 1;
        if (checkWin())
            won = turn;
            
        turn = 3 - turn;
        if (changed != null)
            changed();
        return true;
    }

    public List<Move> possible_moves() {
        var moves = new List<Move>();
        for (int x = 0 ; x < 3 ; ++x)
            for (int y = 0; y < 3 ; ++y)
                if (square[x, y] == 0)
                    moves.Add(new Move(x, y));
        return moves;
    }
}

